@@ -7,7 +7,7 @@ use vars qw($VERSION @EXPORT @EXPORT_OK);
 use base 'Exporter';
 @EXPORT_OK = @EXPORT = qw( hash2xml xml2hash );
 
-$VERSION = '0.53';
+$VERSION = '0.54';
 
 require XSLoader;
 XSLoader::load('XML::Hash::XS', $VERSION);
@@ -15,36 +15,38 @@ XSLoader::load('XML::Hash::XS', $VERSION);
 use vars qw($method $output $root $version $encoding $utf8 $indent $canonical
     $use_attr $content $xml_decl $doc $max_depth $attr $text $trim $cdata
     $comm $buf_size $keep_root $force_array $force_content $merge_text
+    $suppress_empty
 );
 
 # 'NATIVE' or 'LX'
-$method      = 'NATIVE';
+$method         = 'NATIVE';
 
 # native options
-$output        = undef;
-$root          = 'root';
-$version       = '1.0';
-$encoding      = '';
-$utf8          = 1;
-$indent        = 0;
-$canonical     = 0;
-$use_attr      = 0;
-$content       = undef;
-$xml_decl      = 1;
-$keep_root     = 0;#
-$doc           = 0;
-$max_depth     = 1024;
-$buf_size      = 4096;
-$trim          = 0;
-$force_array   = undef;
-$force_content = 0;
-$merge_text    = 0;
+$output         = undef;
+$root           = 'root';
+$version        = '1.0';
+$encoding       = '';
+$utf8           = 1;
+$indent         = 0;
+$canonical      = 0;
+$use_attr       = 0;
+$content        = undef;
+$xml_decl       = 1;
+$keep_root      = 0;#
+$doc            = 0;
+$max_depth      = 1024;
+$buf_size       = 4096;
+$trim           = 0;
+$force_array    = undef;
+$force_content  = 0;
+$merge_text     = 0;
+$suppress_empty = 0;
 
 # XML::Hash::LX options
-$attr          = '-';
-$text          = '#text';
-$cdata         = undef;
-$comm          = undef;
+$attr           = '-';
+$text           = '#text';
+$cdata          = undef;
+$comm           = undef;
 
 1;
 __END__
@@ -222,11 +224,11 @@ if defined that the key name for the text content(used only if use_attr=1).
 
 =item force_array => [ = undef ] I<# xml2hash>
 
-This option is similar to "ForceArray" from XMl::Simple module: L<https://metacpan.org/pod/XML::Simple#ForceArray-1-in---important>.
+This option is similar to "ForceArray" from XMl::Simple module: L<https://metacpan.org/pod/XML::Simple#ForceArray-=%3E-1-%23-in-important>.
 
 =item force_content => [ = 0 ] I<# xml2hash>
 
-This option is similar to "ForceContent" from XMl::Simple module: L<https://metacpan.org/pod/XML::Simple#ForceContent-1-in---seldom-used>.
+This option is similar to "ForceContent" from XMl::Simple module: L<https://metacpan.org/pod/XML::Simple#ForceContent-=%3E-1-%23-in-seldom-used>.
 
 =item merge_text [ = 0 ] I<# xml2hash>
 
@@ -242,6 +244,10 @@ if xml_decl is "0", XML declaration will not be output.
 
 Trim leading and trailing whitespace from text nodes.
 
+=item suppress_empty => [ = 0 ] I<# xml2hash>
+
+This option is similar to "SuppressEmpty" from XMl::Simple module: L<https://metacpan.org/pod/XML::Simple#SuppressEmpty-=%3E-1-%7C-''-%7C-undef-%23-in+out-handy>.
+
 =item utf8 [ = 1 ] I<# hash2xml+xml2hash>
 
 Turn on utf8 flag for strings if enabled.
